class Cell {

  int x,y,h;
  int w,oldW;
  int v,oldV,vMin,vMax;
  int currentC,currentW;
  boolean showOutline = false;
  boolean showMaxOutline = false;
  int maxOutline;
  int minOutline;

  boolean aniVRunning = false;
  boolean aniWRunning = false;
  float aniVDiff,aniV,aniWDiff,aniW;
  int aniVDir,newAniV,aniWDir,newAniW;
  final float aniSpeed = 50;
  
  final int outlineColor = 180;
  final float fillGradientAngle = 97.7;
  final int gradientMin = 2;
  final int gradientMax = 255;

  Cell(int pX, int pY, int pW, int pH, int pV, int pVMin, int pVMax, boolean pShowOutline, boolean pShowMaxOutline, int pMaxOutline) {
    x = pX;
    y = pY;
    w = pW;
    h = pH;
    v = pV;
    vMin = pVMin;
    vMax = pVMax;
    showOutline = pShowOutline;
    showMaxOutline = pShowMaxOutline;
    maxOutline = pMaxOutline;
    minOutline = w;
  }

  void draw() {

    animate();

    pushStyle();
      
      noStroke();
      colorMode(HSB);
      fill(map(fillGradientAngle,0,360,0,255),currentC,255);
      rect(x,y,currentW,h);

      if (showOutline) {               
        stroke(outlineColor);
        noFill();
        if (showMaxOutline) {rect(x,y,maxOutline,h);}
        else {rect(x,y,minOutline,h);} 
      }

    popStyle();

  };

  void animate() {

    // Has the v value changed?
    if (v != oldV) {
      // To avoid jumps when animation is already running
      if (aniVRunning) {oldV = int(aniV);}
      aniV = oldV;
      newAniV = v;
      aniVDiff = v-oldV;
      if (aniVDiff > 0) aniVDir = 1;
      if (aniVDiff < 0) aniVDir = -1;
      aniVRunning = true;
    }

    // Has the w value changed?
    if (w != oldW) {
      // To avoid jumps when animation is already running
      if (aniWRunning) {oldW = int(aniW);}
      aniW = oldW;
      newAniW = w;
      aniWDiff = w-oldW;
      if (aniWDiff > 0) aniWDir = 1;
      if (aniWDiff < 0) aniWDir = -1;
      aniWRunning = true;
    }

    if (aniVRunning) {
      if (aniV >= newAniV && aniVDir == 1 || (aniV <= newAniV && aniVDir == -1))  {aniVRunning = false;}
      aniV += aniVDiff/aniSpeed;
      currentC = int(map(aniV,float(vMin),float(vMax),gradientMin,gradientMax));
    }
    if (!aniVRunning) {
      currentC = int(map(float(v),float(vMin),float(vMax),gradientMin,gradientMax));
    }

    if (aniWRunning) {
      if (aniW >= newAniW && aniWDir == 1 || (aniW <= newAniW && aniWDir == -1))  {aniWRunning = false;}
      aniW += aniWDiff/aniSpeed;
      currentW = int(aniW);
    }
    if (!aniWRunning) {
      currentW = w;
    }

    oldV = v;
    oldW = w;

  };

  boolean mouseOver() {
    if (mouseX > x && mouseX < x+w && mouseY > y && mouseY < y+h) {
      showOutline = true;
      return true;
    }
    else {
      showOutline = false;
      return false;
    }
  }

};


class Checkbox {

  int x,y;
  int w = 10;
  int h = 10;
  boolean status = true;
  color c = color(128);
  String labelOn = "ON";
  String labelOff = "OFF";
  final int textOffset = -1;
  final int labelDistance = 10;

  Checkbox(int pX,int pY,int pW, int pH, color pC, String pLabelOn, String pLabelOff) {
    x = pX;
    y = pY;
    w = pW;
    h = pH;
    c = pC;
    labelOn = pLabelOn;
    labelOff = pLabelOff;
  };

  void draw() {
    pushStyle(); 
      stroke(c);
      noFill();
      textAlign(LEFT,TOP);
      rect(x,y,w,h);
      if (status) {
        line(x,y,x+w,y+h);
        line(x+w,y,x,y+h);
        text(labelOn,x+w+labelDistance,y+textOffset);
      }
      else {
        text(labelOff,x+w+labelDistance,y+textOffset);
      }
    popStyle();
  };

  boolean mouseClicked() {
    if (mouseX > x && mouseX < x+w && mouseY > y && mouseY < y+h) {
      status = !status;
    }
    return status;
  };

};