// Data containers
Table data;
int rows,cols;
int hiddenCol;
int[][] dataMatrix;
int[] maxMatrix;
int[] minMatrix;
Cell[][] cellMatrix;

// Layout parameters
final int cellWidthMax = 100;
final int cellWidthMin = 10;
final int cellHeight = 4;
final int cellXDist = 1;
final int cellYDist = 2;
final int sideDist = 60;
final int middleDist = 30;
final int fontYOffset = -35;
final float labelRelX = 1.7;
final float labelRelY = 2.0;
final int switchSize = 10;
int firstTwo;

// Color parameters
final int bgColor = 255;
final int bgFontColor = 200;
final int labelFontColor = 180;

// Misc.
final int absMax = 2147483647;
final int uDefI = 0;
final boolean uDefB = false;

// Interface
Checkbox checkboxDeweyWarp;
boolean deweyWarp = true;
PFont labelFont,bgFont;
String bgValUpper = "";
String bgValLower = "";


void setup() {
  
  smooth();

  // Set font
  labelFont = createFont("Futura",10);
  bgFont = createFont("Futura",280);
  
  // Load data
  data = loadTable("assignment3_data.csv", "header");
  rows = data.getRowCount();
  cols = data.getColumnCount();
  
  // Hidden column is the last column
  hiddenCol = cols-1;
  
  // Resize arrays
  dataMatrix = new int[rows][cols];
  maxMatrix = new int[cols];
  minMatrix = new int[cols];
  cellMatrix = new Cell[rows][cols];

  // Populate data matrix
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {
      dataMatrix[i][j] = data.getInt(i,j);
    }
  }

  // Prepare minimum and maximum matrices
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {
      maxMatrix[j] = 0;
      minMatrix[j] = absMax;  
    }
  }

  // Populate minimum and maximum matrices
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {
      if (dataMatrix[i][j] < minMatrix[j]) minMatrix[j] = dataMatrix[i][j];
      if (dataMatrix[i][j] > maxMatrix[j]) maxMatrix[j] = dataMatrix[i][j];
    }
  }

  // Populate cell matrix
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {
      cellMatrix[i][j] = new Cell(uDefI,uDefI,uDefI,cellHeight,dataMatrix[i][j],minMatrix[j],maxMatrix[j],uDefB,uDefB,cellWidthMax);
    }
  }

  // Cut the hidden column
  cols-=1;
  
  // Dynamic size, first two columns are always minimum width and followed by middleDist
  firstTwo = sideDist+2*(cellWidthMin+cellXDist)+middleDist;
  size(firstTwo+(cols-2)*(cellWidthMax+cellXDist)+sideDist,rows*(cellHeight+cellYDist)+2*sideDist);
  
  // Checkbox for Dewey warping
  checkboxDeweyWarp = new Checkbox(firstTwo,height-(sideDist/2+switchSize/2),switchSize,switchSize,labelFontColor,"DEWEY WARP ON","DEWEY WARP OFF");
  
};


void draw() {
  
  background(255);
  
  // Draw the background values, position includes hardcoded offset
  textFont(bgFont);
  textAlign(CENTER,CENTER);
  fill(bgFontColor);
  text(bgValUpper,width/2,(height/3)*1+fontYOffset);
  text(bgValLower,width/2,(height/3)*2+fontYOffset);
  
  // Reset background values in each draw cycle
  bgValUpper = "";
  bgValLower = "";
  
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {

      boolean inFirstTwo = false;
      if (j==0 || j==1) {inFirstTwo = true;}
        
      // If we're not in the first two columns and Dewey warping is on, use relative width
      if (!inFirstTwo && deweyWarp) {
        cellMatrix[i][j].w = int(map(float(dataMatrix[i][hiddenCol]),float(minMatrix[hiddenCol]),float(maxMatrix[hiddenCol]),cellWidthMin,cellWidthMax));
        cellMatrix[i][j].x = firstTwo+(j-2)*(cellWidthMax+cellXDist);
      }
      // If we're not in the first two columns and Dewey warping is off, use maximum width
      else if (!inFirstTwo && !deweyWarp) {
        cellMatrix[i][j].w = cellWidthMax;
        cellMatrix[i][j].x = firstTwo+(j-2)*(cellWidthMax+cellXDist);
      }
      // If we're in the first two columns and Dewey warping is on or off, use minimum width
      else {
        cellMatrix[i][j].w = cellWidthMin;
        cellMatrix[i][j].x = sideDist+j*(cellWidthMin+cellXDist);
      }
        
      cellMatrix[i][j].y = sideDist+i*(cellHeight+cellYDist);
      cellMatrix[i][j].v = dataMatrix[i][j];
        
      if (cellMatrix[i][j].mouseOver()) {
        bgValUpper = str(dataMatrix[i][j]);       
        // Show the lower values and the maximum outline only if we're not in the first two columns
        if (!inFirstTwo) {
          bgValLower = str(dataMatrix[i][hiddenCol]);
          cellMatrix[i][j].showMaxOutline = true;
        }
        else {
          cellMatrix[i][j].showMaxOutline = false;    
        }
      }

      cellMatrix[i][j].draw();
  
    }
  }
  
  textFont(labelFont);
  fill(labelFontColor);
  
  int labelOffset;
  if (deweyWarp) {
    textAlign(LEFT,CENTER);
    labelOffset = firstTwo;
  }
  else {
    textAlign(CENTER,CENTER);
    labelOffset = firstTwo+cellWidthMax/2;
  }
  
  // Horizontal labels
  text("MARX", labelOffset,sideDist/labelRelX);
  text("KEYNES", labelOffset+1*(cellWidthMax+cellXDist),sideDist/labelRelX);
  text("CAPITAL", labelOffset+2*(cellWidthMax+cellXDist),sideDist/labelRelX);
  text("CRISIS", labelOffset+3*(cellWidthMax+cellXDist),sideDist/labelRelX);
  
  // Vertical labels
  pushMatrix();
    translate(sideDist/labelRelY,height/2);
    rotate(radians(270));
    textAlign(CENTER,CENTER);
    text("FOUR RECESSION RELATED KEYWORDS IN TITLES 2006-2014 AND TOTAL TITLES FOR DEWEY DECIMALS 330-339: ECONOMICS",0,0);
  popMatrix();
  
  checkboxDeweyWarp.draw();
  
};


void mouseClicked() {
  deweyWarp = checkboxDeweyWarp.mouseClicked();
};


// Java passes arrays as reference!
void bubbleSort(int[][] a, int c) {

  boolean s = true;

  while (s == true) {
    s = false;
    for (int n=0; n<rows-1; n++) {
      // Compare the values in just one column, but swap values in all columns, including the hidden column
      if (a[n][c] > a[n+1][c]) {
        for (int r=0; r<cols+1; r++) {
          int t = a[n][r];
          a[n][r] = a[n+1][r];
          a[n+1][r] = t;
        }
        s = true;
      }
    }
  }

};


void keyPressed() {

  if (key == '1') {bubbleSort(dataMatrix,0);}
  if (key == '2') {bubbleSort(dataMatrix,1);}
  if (key == '3') {bubbleSort(dataMatrix,2);}
  if (key == '4') {bubbleSort(dataMatrix,3);}
  if (key == '5') {bubbleSort(dataMatrix,4);}
  if (key == '6') {bubbleSort(dataMatrix,5);}
  if (key == '7') {bubbleSort(dataMatrix,6);}

};